@@ -31,7 +31,10 @@ from mercurial import (
 from mercurial.hgweb import webutil, hgweb_mod
 from mercurial.hgweb.common import paritygen
 
-import cmarkgfm
+import mistune
+
+from mistune.directives import DirectiveToc
+
 
 filters = {}
 templatefilter = registrar.templatefilter(filters)
@@ -241,6 +244,17 @@ def _diffsgen(
         yield data
 
 
+def http_clone_url(context, mapping):
+    """ http_clone_url is a template keyword that outputs the configuration
+        value of `web.http_clone_url`.  It can be used in themes to make it
+        easier for users to clone repositories.
+    """
+
+    ui = context.resource(mapping, b'ui')
+
+    return ui.config(b'web', b'http_clone_url', b'')
+
+
 def readme(context, mapping):
     """ readme is a template keyword that will attempt to render a readme file
         that is available in the repository.  It currently supports github
@@ -249,6 +263,12 @@ def readme(context, mapping):
 
     ctx = context.resource(mapping, b'ctx')
 
+    class Renderer(mistune.HTMLRenderer):
+        def image(self, src, alt, title):
+            src = 'rawfile/%s/%s' % (ctx.rev(), src)
+
+            return super().image(src, alt, title)
+
     # we iterate the files instead of a fileset, because we want to
     # deterministically render readmes in the same order if there are more
     # than one in a repository.  With the fileset, we'd have to run through
@@ -258,11 +278,57 @@ def readme(context, mapping):
         lower = filename.lower()
         if lower in [b'readme.txt', b'readme', b'readme.md']:
             raw_utf8 = ctx[filename].data().decode('utf-8')
-            return cmarkgfm.markdown_to_html(raw_utf8).encode('utf-8')
+
+            plugins = [
+                DirectiveToc(),
+                'footnotes',
+                'strikethrough',
+                'table',
+                'url',
+            ]
+
+            markdown = mistune.create_markdown(
+                renderer=Renderer(),
+                plugins=plugins,
+            )
+
+            return markdown(raw_utf8).encode('utf-8')
 
     return ""
 
 
+def ssh_clone_url(context, mapping):
+    """ ssh_clone_url is a template keyword that outputs the configuration
+        value of `web.ssh_clone_url`.  It can be used in themes to make it
+        easier for users to clone repositories.
+    """
+
+    ui = context.resource(mapping, b'ui')
+
+    return ui.config(b'web', b'ssh_clone_url', b'')
+
+
+def sub_title(context, mapping):
+    """ sub_title is a template keyword that outputs the configuration value of
+        `web.sub_title`.  Themes can use this as a secondary text to the title.
+    """
+
+    ui = context.resource(mapping, b'ui')
+
+    return ui.config(b'web', b'sub_title', b'')
+
+
+def title(context, mapping):
+    """ title is a template keyword that outputs the configuration value of
+        `web.title`.  Themes can use this as the HTML title and else where on
+        the pages.
+    """
+
+    ui = context.resource(mapping, b'ui')
+
+    return ui.config(b'web', b'title', b'')
+
+
 def repo_custom_templater(orig, self, req):
     """ this function is a wrapper of hgweb_mod.requestcontext.templater.  This
         is done to be able to add custom keywords into hgweb.  This is not an
@@ -273,7 +339,11 @@ def repo_custom_templater(orig, self, req):
 
     templatekeyword = registrar.templatekeyword(templater._proc._defaults)
 
+    templatekeyword(b'http_clone_url', requires={b'ui'})(http_clone_url)
     templatekeyword(b'readme', requires={b'ctx', b'repo'})(readme)
+    templatekeyword(b'ssh_clone_url', requires={b'ui'})(ssh_clone_url)
+    templatekeyword(b'sub_title', requires={b'ui'})(sub_title)
+    templatekeyword(b'title', requires={b'ui'})(title)
 
     return templater
 
